program login;

{$mode objfpc}

uses
  browserapp, JS, Classes, SysUtils, web, Webwidget, htmlwidgets, services,
  fprpcclient, weborworker;

type

  { TMyApplication }

  TMyApplication = class(TBrowserApplication)
    edtEmail: TTextInputWidget;
    edtPassword : TTextInputWidget;
    edt2FA : TTextInputWidget;
    btnLogin : TButtonWidget;
    btnContinue : TButtonWidget;
    divLogin : TTagWidget;
    div2FA : TTagWidget;
    divError : TTagWidget;
    divMenuLogin : TTagWidget;
    divMenuLogout : TTagWidget;
    divMenuHamburger : TTagWidget;
    divMenu : TTagWidget;
    divdlgLogin : TTagWidget;
    divMenuUser : TTagWidget;
    divLblUser : TTagWidget;
    procedure doRun; override;
  private
    FLoggedInUser : String;
    FRPCClient : TRPCClient;
    FUserService : TUserService;
    procedure BindElements;
    procedure ClickNavBar(Sender: TObject; Event: TJSEvent);
    procedure doContinueClick(sender: TObject; event: TJSEvent);
    procedure doLoginClick(sender : TObject; event : TJSEvent);
    procedure DoLoginMenuClick(Sender: TObject; Event: TJSEvent);
    procedure DoLogout;
    procedure DoLogoutClick(Sender: TObject; Event: TJSEvent);
    procedure doServerLogin(const aUser, aPassword: String; aCode : Integer);
    procedure SetupServices;
    procedure ShowLogin(const aError: String);
    procedure ShowLogout;
    procedure StartLogin(const aUser: String);
  end;


procedure TMyApplication.BindElements;
begin
  edtEmail:=TTextInputWidget.Create(Self);
  edtEmail.elementID:='edtEmail';
  edtEmail.Refresh;
  edtPassword:=TTextInputWidget.Create(Self);
  edtPassword.ElementID:='edtPassword';
  edtPassword.Refresh;
  edt2FA:=TTextInputWidget.Create(Self);
  edt2FA.ElementID:='edt2FA';
  edt2FA.Refresh;
  btnLogin:=TButtonWidget.Create(Self);
  btnLogin.ElementID:='btnLogin';
  btnLogin.Text:='Login <i class="fa fa-sign-in" aria-hidden="true"></i>';
  btnLogin.TextMode:=tmHTML;
  btnLogin.Refresh;
  btnContinue:=TButtonWidget.Create(Self);
  btnContinue.ElementID:='btnContinue';
  btnContinue.Text:='Continue <i class="fa fa-sign-in" aria-hidden="true"></i>';
  btnContinue.TextMode:=tmHTML;
  btnContinue.Refresh;
  divLogin:=TTagWidget.Create(Self);
  divLogin.ElementID:='login';
  divLogin.Refresh;
  div2FA:=TTagWidget.Create(Self);
  div2FA.ElementID:='2FA';
  div2FA.Refresh;
  divError:=TTagWidget.Create(Self);
  divError.ElementID:='error';
  divError.Refresh;
  divMenuLogin:=TTagWidget.Create(Self);
  divMenuLogin.elementID:='mnuLogin';
  divMenuLogin.OnClick:=@DoLoginMenuClick;
  divMenuLogin.Refresh;
  divMenuLogout:=TTagWidget.Create(Self);
  divMenuLogout.elementID:='mnuLogout';
  divMenuLogout.Refresh;
  divMenuLogout.OnClick:=@DoLogoutClick;
  divMenuHamburger:=TTagWidget.Create(Self);
  divMenuHamburger.elementID:='navbar-burger';
  divMenuHamburger.Refresh;
  divMenuHamburger.OnClick:=@ClickNavBar;
  divMenu:=TTagWidget.Create(Self);
  divMenu.elementID:='navbar-main';
  divMenu.Refresh;
  divdlgLogin:=TTagWidget.Create(Self);
  divdlgLogin.elementID:='dlgLogin';
  divdlgLogin.Refresh;
  divMenuUser:=TTagWidget.Create(Self);
  divMenuUser.elementID:='mnuUser';
  divMenuUser.Refresh;
  divLblUser:=TTagWidget.Create(Self);
  divLblUser.elementID:='lblUser';
  divLblUser.Refresh;
end;

procedure TMyApplication.ClickNavBar(Sender: TObject; Event: TJSEvent);
begin
  if Pos('is-active',divMenu.Classes)<>0 then
    DivMenu.RemoveClasses('is-active')
  else
    DivMenu.AddClasses('is-active');
end;

procedure TMyApplication.doContinueClick(sender : TObject; event : TJSEvent);

begin
  divLogin.AddClasses('is-hidden');
  div2FA.RemoveClasses('is-hidden');
end;

procedure TMyApplication.doLoginClick(sender : TObject; event : TJSEvent);

begin
  DoServerLogin(edtEmail.Value,edtPassword.Value, StrToIntDef(edt2FA.Value,0))
end;

procedure TMyApplication.DoLoginMenuClick(Sender: TObject; Event: TJSEvent);
begin
  divDlgLogin.RemoveClasses('is-hidden');
  ShowLogin('');
end;

procedure TMyApplication.DoLogout;

begin
  FUserService.Logout();
end;

procedure TMyApplication.DoLogoutClick(Sender: TObject; Event: TJSEvent);
begin
  ShowLogout;
  DoLogout
end;

procedure TMyApplication.ShowLogout;

begin
  divDlgLogin.RemoveClasses('is-hidden');
  ShowLogin('');
  divMenuUser.AddClasses('is-hidden');
  divMenuLogin.RemoveClasses('is-hidden');
  divMenuLogout.AddClasses('is-hidden');
end;

procedure TMyApplication.ShowLogin(const aError : String);

begin
  div2FA.AddClasses('is-hidden');
  divLogin.RemoveClasses('is-hidden');
  if aError<>'' then
    begin
    divError.RemoveClasses('is-hidden');
    divError.TextContent:=aError;
    end
  else
    begin
    divError.AddClasses('is-hidden');
    divError.TextContent:='';
    end;

end;

procedure TMyApplication.StartLogin(Const aUser : String);

begin
  divdlgLogin.AddClasses('is-hidden');
  divMenuLogout.RemoveClasses('is-hidden');
  divMenuLogin.AddClasses('is-hidden');
  divMenuUser.RemoveClasses('is-hidden');
  divLblUser.TextContent:=aUser;
  FLoggedInUser:=aUser;
end;



procedure TMyApplication.doServerLogin(const aUser,aPassword: String; aCode : Integer);

  procedure DoOK(aResult: Boolean);
  begin
    if not aResult then
      ShowLogin('Invalid combination of username/password')
    else
      StartLogin(aUser);
  end;

  procedure DoFail(Sender: TObject; const aError: TRPCError);

  begin
    ShowLogin('Error during login: '+aError.Message);
  end;


begin
  FUserService.Login(aUser,aPassword,aCode,@DoOK,@DoFail);
end;

procedure TMyApplication.SetupServices;

begin
  FRPCClient:=TRPCClient.Create(Self);
  FRPCClient.URL:='/RPC';
  FUserService:=TUserService.Create(Self);
  FUserService.RPCClient:=FRPCClient;
end;

procedure TMyApplication.doRun;

begin
  BindElements;
  SetupServices;
  btnContinue.OnClick:=@DoContinueClick;
  btnLogin.OnClick:=@DoLoginClick;
  Terminate;
end;

var
  Application : TMyApplication;

begin
  Application:=TMyApplication.Create(nil);
  Application.Initialize;
  Application.Run;
end.
